# -*- coding: utf-8 -*-

# mapWOC
#
# Description: JSON Tests.
#
# Authors:
# Felix Wolfsteller <felix.wolfsteller@intevation.de>
#
# Copyright:
# Copyright (C) 2011 Bundesamt für Sicherheit in der Informationstechnik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2,
# or, at your option, any later version as published by the Free
# Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

import json
import unittest

from mapwoc.lib.hc import HC
from mapwoc.lib.hccontrol import HCControl
from mapwoc.lib.message import Message
from mapwoc.lib.redirectorvisit import RedirectorVisit
from mapwoc.lib.scanconfiguration import VisitorBundle, VisitorConfiguration
from mapwoc.lib.trivialresponse import TrivialResponse
from mapwoc.lib.url import URL

from mapwoc.lib.mapwoc_json import MapWOCJSONEncoder, mapwoc_from_json_hook

def pretty_print(objs):
    return json.dumps(objs, sort_keys=True, indent=2, cls=MapWOCJSONEncoder)

def to_json_string(objs):
    return MapWOCJSONEncoder().encode(objs)

def from_json_string(jsonstring):
    return json.loads(jsonstring, object_hook=mapwoc_from_json_hook)

class JSONComparerTestCase(unittest.TestCase):
    def compare_objs(self, obj1, obj2):
        """
        Compares the attributes of two objects.
        Difference to modules compareObjs is that comparisons are done
        with assertEqual directly.

        @param obj1: First object to compare.
        @param obj2: Object to compare first object against.
        @return True if they are equal, False if they are not equal.
        Note that the test checks whether all attributes of obj1 can be
        found in obj2.  Thus, compareObjs(o1,o2) might evaluate to True,
        but compareObjs(o2,o1) might not (i.e. o2 specifies more
        attributes than o1).
        """
        for attr in dir(obj1):
            if not attr.startswith("__") and not callable(getattr(obj1, attr)):
                self.assertEqual(getattr(obj1, attr), getattr(obj2, attr), "%s does not match" % attr)
                return False
        return True

class TestTrivialResponse(JSONComparerTestCase):

    def test_EmptyResponse(self):
        tr1 = TrivialResponse()
        jstr1 = to_json_string(tr1)
        job1 = from_json_string(jstr1)
        self.compare_objs(tr1, job1)

    def test_NoneResponse(self):
        tr2 = TrivialResponse(None, None)
        jstr2 = to_json_string(tr2)
        job2 = from_json_string(jstr2)
        self.compare_objs(tr2, job2)

    def test_ErrorMessage(self):
        tr3 = TrivialResponse(1, "This is an Error!")
        jstr3 = to_json_string(tr3)
        job3 = from_json_string(jstr3)
        self.compare_objs(tr3, job3)

class TestVisitorBundle(JSONComparerTestCase):

    def test_VisitorConfiguration(self):
        vb1 = VisitorBundle("abc", "def", "geh")
        vb2 = VisitorBundle("123", "456", "789")
        vc = VisitorConfiguration()
        vc.visitors += [vb1]
        vc.visitors += [vb2]
        vc.eventmap = "7e10h"

        jstr = to_json_string(vc)
        job = from_json_string(jstr)
        self.compare_objs(vc, job)

class TestHCS(JSONComparerTestCase):

    def test_hcOneNode(self):
        hc = HC()
        hc.name = "testhc"
        hc.description = "testdescription"
        hc.hcsid = "hctest1"
        hc.active_browser = "cpe://a:microsoft:ie6"
        hc.on_nodes = ["Node1"]

        jstr = to_json_string(hc)
        job = from_json_string(jstr)
        self.compare_objs(hc, job)

    def test_hcTwoNodes(self):
        hc = HC()
        hc.name = "testhc"
        hc.description = "testdescription"
        hc.hcsid = "hctest1"
        hc.active_browser = "cpe://a:microsoft:ie6"
        hc.on_nodes = ["Node1", "Node2"]

        jstr = to_json_string(hc)
        job = from_json_string(jstr)
        self.compare_objs(hc, job)

class TestHCControls(JSONComparerTestCase):

    def test_hccontrol1(self):
        hc = HCControl()
        hc.name = "testhc"
        hc.description = "testdescription"
        hc.hcsid = "hctest1"
        hc.active_browser = "cpe://a:microsoft:ie6"
        hc.on_nodes = ["Node1", "Node2"]
        hc.image_memory = 128
        hc.image_snapshot = "/home/to/snap.shot"
        hc.installed_software = "cpe:a:b:c;cpe:a:d:e"
        hc.os = "cpe:o:s"
        hc.spe = "be"

        jstr = to_json_string(hc)
        job = from_json_string(jstr)
        self.compare_objs(hc, job)

class TestMessage(JSONComparerTestCase):

    def test_simple(self):
        me = Message(Message.LEVEL_RESULT, "test results")

        jstr = to_json_string(me)
        job = from_json_string(jstr)
        self.compare_objs(me, job)


class TestRedirectorVisit(JSONComparerTestCase):

    def test_simple(self):
        rv = RedirectorVisit('hcsid', 'http://url.de', '10.10.10.10')

        jstr = to_json_string(rv)
        job = from_json_string(jstr)
        self.compare_objs(rv, job)

class TestURL(JSONComparerTestCase):
    def test_simple(self):
        ur = URL('https://urls.de')

        jstr = to_json_string(ur)
        job = from_json_string(jstr)
        self.compare_objs(ur, job)

